SystemOrganization addCategory: #'Stack-Exercise'!

Object subclass: #OOStack
	instanceVariableNames: 'list'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!OOStack class methodsFor: 'error descriptions' stamp: 'HernanWilkinson 5/7/2012 11:51'!
stackEmptyErrorDescription
	
	^ 'Stack is empty'! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'PabloFromer 5/19/2012 21:13'!
initialize 

	list:= OrderedCollection new.
	list add: BaseStackElement new.
	
	! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'PabloFromer 5/19/2012 20:45'!
isEmpty 

	^ (list size = 1).! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'PabloFromer 5/19/2012 20:44'!
pop

	|elementToReturn|
	
	elementToReturn := self top.
	
	list removeLast.
	
	^ elementToReturn.! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'PabloFromer 5/19/2012 21:12'!
push: anObject
	
	|noBaseStackElement |

	noBaseStackElement := NoBaseStackElement new.
	noBaseStackElement setWrappedElement:  anObject.
	
	list add: noBaseStackElement. 
		! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'PabloFromer 5/19/2012 21:14'!
size
	
	^list size - 1.! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'PabloFromer 5/19/2012 20:40'!
top
	
	|listLastElement|
	
	listLastElement := list last.
	
	^listLastElement ifYouAreBaseReturnErrorOtherwiseGiveMeYourWrappedElement.		
! !

Object subclass: #StackElement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

StackElement subclass: #BaseStackElement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!BaseStackElement methodsFor: 'as yet unclassified' stamp: 'PabloFromer 5/19/2012 21:15'!
ifYouAreBaseReturnErrorOtherwiseGiveMeYourWrappedElement

	self error: 'Stack is empty'.! !

StackElement subclass: #NoBaseStackElement
	instanceVariableNames: 'WrappedElement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!NoBaseStackElement methodsFor: 'as yet unclassified' stamp: 'PabloFromer 5/19/2012 20:42'!
ifYouAreBaseReturnErrorOtherwiseGiveMeYourWrappedElement

	^WrappedElement.! !

!NoBaseStackElement methodsFor: 'as yet unclassified' stamp: 'PabloFromer 5/19/2012 20:33'!
setWrappedElement: anObject.

	WrappedElement := anObject. ! !

!NoBaseStackElement methodsFor: 'as yet unclassified' stamp: 'PabloFromer 5/19/2012 20:24'!
wrappedElement

	^WrappedElement.! !

!StackElement methodsFor: 'as yet unclassified' stamp: 'PabloFromer 5/19/2012 20:40'!
ifYouAreBaseReturnErrorOtherwiseGiveMeYourWrappedElement

	self subclassResponsibility.! !

TestCase subclass: #OOStackTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!OOStackTest methodsFor: 'test' stamp: 'PabloFromer 5/19/2012 21:17'!
test01StackShouldBeEmptyWhenCreated

	| stack |
	
	self halt.
	
	stack := OOStack new.
	
	self assert: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:30'!
test02PushAddElementsToTheStack

	| stack |
	
	stack := OOStack new.
	stack push: 'Something'.
	
	self deny: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:31'!
test03PopRemovesElementsFromTheStack

	| stack |
	
	stack := OOStack new.
	stack push: 'Something'.
	stack pop.
	
	self assert: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/8/2012 08:20'!
test04PopReturnsLastPushedObject

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'Something'.
	stack push: pushedObject.
	
	self assert: stack pop = pushedObject! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:33'!
test05StackBehavesLIFO

	| stack firstPushedObject secondPushedObject |
	
	stack := OOStack new.
	firstPushedObject := 'First'.
	secondPushedObject := 'Second'.
	
	stack push: firstPushedObject.
	stack push: secondPushedObject.
	
	self assert: stack pop = secondPushedObject.
	self assert: stack pop = firstPushedObject.
	self assert: stack isEmpty 
	! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:35'!
test06TopReturnsLastPushedObject

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'Something'.
	
	stack push: pushedObject.
	
	self assert: stack top = pushedObject.
	! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:36'!
test07TopDoesNotRemoveObjectFromStack

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'Something'.
	
	stack push: pushedObject.
	
	self assert: stack size = 1.
	stack top.
	self assert: stack size = 1.
	! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:38'!
test08CanNotPopWhenThereAreNoObjectsInTheStack

	| stack  |
	
	stack := OOStack new.
	self
		should: [ stack pop ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = OOStack stackEmptyErrorDescription ]
		
! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:39'!
test09CanNotTopWhenThereAreNoObjectsInTheStack

	| stack  |
	
	stack := OOStack new.
	self
		should: [ stack top ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = OOStack stackEmptyErrorDescription ]
		
! !
